@@ -54,7 +54,8 @@ ArenaBlock* SabArena::newBlock(int size)
     totalMalloced += size;
     ArenaBlock *b = new ArenaBlock;
     b -> next = NULL;
-    b -> data = malloc(b -> freeSpace = size);   // guaranteed to be well-aligned
+    b -> data = malloc(b -> freeSpace = size); // guaranteed to be well-aligned
+    b -> blockSize = size;
     return b;
 }
 
@@ -85,6 +86,7 @@ void* SabArena::armalloc(int size, int alignment /* = sizeof(void*) */)
     int thisBloxize = bloxize;
     if (!block0)
         block0 = blockn = newBlock(bloxize);
+    thisBloxize = blockn -> blockSize;
     if ((blockn -> freeSpace &= ~(alignment - 1)) < size) // align as necessary
     {
         if (size > bloxize)
