@@ -131,6 +131,7 @@ const char* theXMLNamespace = "http://www.w3.org/XML/1998/namespace";
 const char* theXHTMLNamespace = "http://www.w3.org/1999/xhtml";
 const char* theXMLNSNamespace = "http://www.w3.org/2000/xmlns/";
 const char* theSabExtNamespace = "http://www.gingerall.org/sablotron/extension";
+const char* theEXSLTDynNamespace = "http://exslt.org/dynamic";
 
 const char* theWhitespace = " \t\x0a\x0d";
 
@@ -377,18 +378,27 @@ Bool isValidNCName(const char* name)
 Bool isValidQName(const char* name)
 {
   char *local = NULL;
+  char *start = NULL;
+  Bool copy = false;
   Bool result = TRUE;
 
   char *colon = (char *)strchr(name, ':');
   if (colon) 
     {
-      *colon = '\0';
+      //*colon = '\0';
       local = colon + 1;
+      copy = true;
+      start = new char[colon - name + 1];
+      strncpy(start, name, colon - name);
+      start[colon - name] = '\0';
+    } else {
+      start = (char*)name;
     }
 
-  result = isValidNCName(name) && (local == NULL || isValidNCName(local));
+  result = isValidNCName(start) && (local == NULL || isValidNCName(local));
 
-  if (colon) *colon = ':';
+  //if (colon) *colon = ':';
+  if (copy) delete start;
   return result;
 }
 
