@@ -247,6 +247,10 @@ enum ExFunctor
     //XSLT extensions
         EXFF_FUNCTION_AVAILABLE,
         EXFF_ELEMENT_AVAILABLE,
+
+    //GA extensions
+    EXFF_EVAL,
+
         EXFF_NONE
 };
 
