@@ -128,7 +128,13 @@ JSContextItem::~JSContextItem()
   if (errInfo.token) delete errInfo.token;
   if (cx) 
     {
+#ifdef ENABLE_JS_THREADS
+      JS_ResumeRequest(cx);
+#endif
       if (array_proto) JS_RemoveRoot(cx, &array_proto);
+#ifdef ENABLE_JS_THREADS
+      JS_EndRequest(cx);
+#endif
       JS_GC(cx);
       JS_DestroyContext(cx);
     }
@@ -157,6 +163,9 @@ JSContextItem* JSContextList::find(Str uri, Bool canCreate /*TRUE*/)
     {
       JSContext_Sab *cx;
       cx = JSManager::createContext();
+#ifdef ENABLE_JS_THREADS
+      JS_BeginRequest(cx);
+#endif
       item = new JSContextItem(cx, uri, proc);
       append(item);
       //initalize context
@@ -171,6 +180,9 @@ JSContextItem* JSContextList::find(Str uri, Bool canCreate /*TRUE*/)
       JS_SetErrorReporter(cx, SJSErrorReporter);
       //create persistent objects
       jsdom_delegateDOM(cx);
+#ifdef ENABLE_JS_THREADS
+      JS_Suspendrequest(cx);
+#endif
     }
   return item;
 }
@@ -239,6 +251,10 @@ Bool SJSEvaluate(JSContextItem &item, DStr &script)
   JSBool status;
   JSContext *cx = item.cx;
 
+#ifdef ENABLE_JS_THREADS
+  JS_ResumeRequest(cx);
+#endif
+
   /*
     int len = utf8StrLength((const char*) script);
     wchar_t *ucscript = new wchar_t[len + 1];
@@ -286,6 +302,11 @@ Bool SJSEvaluate(JSContextItem &item, DStr &script)
   else { //error occured
     
   }
+
+#ifdef ENABLE_JS_THREADS
+  JS_SuspendRequest(cx);
+#endif
+
   return (Bool)status;
 }
 
@@ -440,6 +461,10 @@ Bool SJSCallFunction(Sit S, Context *c, JSContextItem &item, Str &name,
   JSContext *cx = item.cx;
   sabassert(cx);
 
+#ifdef ENABLE_JS_THREADS
+  JS_ResumeRequest(cx);
+#endif
+
   sjs_PublishContext(S, cx, c);
 
   //get args
@@ -597,6 +622,11 @@ Bool SJSCallFunction(Sit S, Context *c, JSContextItem &item, Str &name,
 	retxpr.setAtom(rstr);
       }
   }
+
+#ifdef ENABLE_JS_THREADS
+  JS_SuspendRequest(cx);
+#endif
+
   //run GC
   JS_MaybeGC(cx);
   //JS_GC(cx);
