@@ -271,7 +271,7 @@ eFlag OutputDefinition::setItemStr(Sit S, XSL_ATT itemId, const Str& value, Vert
     }; break;
     };
     int index = lookupAttCode(outputStringAtts, itemId);
-    assert(index >= 0);
+    sabassert(index >= 0);
     if (stringItems[index].set(value, precedence))
     {
 	S.setCurrVDoc(caller);
@@ -290,7 +290,7 @@ eFlag OutputDefinition::setItemEQName(Sit S, XSL_ATT itemId, const EQName& value
         cdataElems.append(new EQName(value)); break;
     default: 
     {
-        assert(itemId == XSLA_METHOD);
+        sabassert(itemId == XSLA_METHOD);
 	if (method.set(value, precedence))
 	{
 	    S.setCurrVDoc(caller);
@@ -304,19 +304,19 @@ eFlag OutputDefinition::setItemEQName(Sit S, XSL_ATT itemId, const EQName& value
 const Str& OutputDefinition::getValueStr(XSL_ATT itemId) const
 {
     int index = lookupAttCode(outputStringAtts, itemId);
-    assert(index >= 0);
+    sabassert(index >= 0);
     return stringItems[index].get();
 }
 
 const EQName& OutputDefinition::getValueEQName(XSL_ATT itemId) const
 {
-    assert(itemId == XSLA_METHOD);
+    sabassert(itemId == XSLA_METHOD);
     return method.get();
 };
 
 Bool OutputDefinition::askEQNameList(XSL_ATT itemId, const EQName &what) const
 {
-    assert(itemId == XSLA_CDATA_SECT_ELEMS);
+    sabassert(itemId == XSLA_CDATA_SECT_ELEMS);
     return (cdataElems.find(what) != NULL);
 }
 
@@ -348,7 +348,7 @@ Bool OutputDefinition::getIndent() const
 eFlag OutputDefinition::setDefaults(Sit S)
 {
     OutputMethod meth = getMethod();
-    assert( meth != OUTPUT_UNKNOWN );
+    sabassert( meth != OUTPUT_UNKNOWN );
     char strYes[] = "yes", strNo[] = "no";
     E( setItemStr(S, XSLA_ENCODING, "UTF-8", NULL, OUTPUT_PRECEDENCE_WEAKEST) );
     switch(meth)
@@ -487,12 +487,12 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
       //prefix = &(namespaces[i] -> prefix); 
       //if (!namespaces.isHidden(*prefix))
 
-      // swallow hidden namespaces as well as the empty default
+      //swallow hidden namespaces as well as the empty default
       //namespace, if no default namespace was output recently
       if (! nm -> hidden &&
 	  (!(prefix -> isEmpty()) || !(nm -> uri.isEmpty()) || defaultNSWas))
 	{
-	  defaultNSWas = prefix -> isEmpty();
+	  defaultNSWas = defaultNSWas || prefix -> isEmpty();
 
           sendLit(S, " xmlns");
           if (!prefix -> isEmpty())
@@ -874,11 +874,10 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
                 case ESCAPING_ATTR:
                 case ESCAPING_LT_AMP:
                     {
-		                if (method == OUTPUT_HTML || method == OUTPUT_XHTML)
-				        {
-                            E( sendLit(S, "&gt;") );
-                            served = TRUE;
-			            }
+		      //now we escape &gt; always to prevent bad
+		      //escaping in cdata sections.
+		      E( sendLit(S, "&gt;") );
+		      served = TRUE;
                     }; break;
                 }
             }; break;
@@ -965,7 +964,7 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
             }; break;
 	    default:
 	        {
-		        // escape non-ASCII values in URIs
+		  // escape non-ASCII values in URIs
 		  if (*data & 0x80)
 		    {
 		      switch(escapeMode)
@@ -988,7 +987,7 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
         if (!served)
         {
             srcCharLen = utf8SingleCharLength(data);
-            assert(srcCharLen > 0);
+            sabassert(srcCharLen > 0);
             // if encoding differs from utf-8 then recode
 
 #ifdef SABLOT_DEBUGGER
@@ -1016,7 +1015,7 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
                 curr = outbuf - buffer;
                 // check result, write a character code if 
 		//couldn't convert (unless ESCAPING_NONE)
-                assert(result != ENC_EINVAL && result != ENC_E2BIG);
+                sabassert(result != ENC_EINVAL && result != ENC_E2BIG);
                 // FIXME: in NT, must check differently
                 if (result == ENC_EILSEQ)
                 {
@@ -1054,7 +1053,7 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
 eFlag PhysicalOutputLayerObj::setMethodByDefault(Sit S, OutputMethod method_)
 {
     EQName q;
-    assert(method == OUTPUT_UNKNOWN);
+    sabassert(method == OUTPUT_UNKNOWN);
     switch( method = method_ )
     {
     case OUTPUT_XML:
@@ -1062,13 +1061,23 @@ eFlag PhysicalOutputLayerObj::setMethodByDefault(Sit S, OutputMethod method_)
     case OUTPUT_HTML:
         q.setLocal("html"); break;
     default:
-        assert(!"PhysicalOutputLayerObj::setMethod()");
+        sabassert(!"PhysicalOutputLayerObj::setMethod()");
     }
     E( NZ(outDef) -> setItemEQName(S, XSLA_METHOD, q, NULL, OUTPUT_PRECEDENCE_STRONGEST) );
     E( outDef -> setDefaults(S) );
     return OK;
 }
 
+eFlag PhysicalOutputLayerObj::close(Sit S) 
+{
+  DataLine * dl;
+  if (dl = getDataLine())
+    {
+      E( dl -> close(S) );
+    }
+  return OK;
+}
+
 void PhysicalOutputLayerObj::report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2)
 {
     S.message(type, code, arg1, arg2);
@@ -1110,7 +1119,9 @@ void FrontMatter::report(Sit S, MsgType type, MsgCode code, const Str& arg1, con
 
 OutputDocument::~OutputDocument()
 {
-  if (outputter) delete outputter;
+  //we do not delete outputter objects as they are stored with
+  //Processor
+  // if (outputter) delete outputter;
   if (def) delete def;
 }
 
@@ -1119,7 +1130,7 @@ eFlag OutputDocument::finish(Sit S)
   if (state == OUTDOC_ACTIVE)
     {
       E( NZ(outputter) -> eventTrailingNewline(S) );
-      E( NZ(outputter) -> eventEndOutput(S) );
+      E( NZ(outputter) -> eventEndOutput(S, true) );
       setState(OUTDOC_FINISHED);
     }
   return OK;
@@ -1270,7 +1281,7 @@ eFlag OutputterObj::reportXMLDeclIfMust(Sit S)
 eFlag OutputterObj::reportDTDIfMust(Sit S, const EQName& docElementName)
 // to be called only after the output method is determined
 {
-    assert(method != OUTPUT_TEXT);
+    sabassert(method != OUTPUT_TEXT);
     if (!physical)
         return OK;
     const Str& 
@@ -1306,7 +1317,7 @@ eFlag OutputterObj::reportDTDIfMust(Sit S, const EQName& docElementName)
 
 eFlag OutputterObj::reportFront(Sit S)
 {
-    assert(method != OUTPUT_UNKNOWN);
+    sabassert(method != OUTPUT_UNKNOWN);
     int i, frontCount = front.number();
     FrontMatterItem *item;
     for(i = 0; i < frontCount; i++)
@@ -1413,7 +1424,7 @@ eFlag OutputterObj::eventElementStart(Sit S, const EQName& name)
     case STATE_IN_ATTRIBUTE:
       Err( S, E_ELEM_IN_COMMENT_PI );
     default:
-      assert(!"eventElementStart");
+      sabassert(!"eventElementStart");
     };
   
   // determine whether we're in a HEAD html element so a META tag
@@ -1477,7 +1488,7 @@ eFlag OutputterObj::throwInMeta(Sit S)
 
 eFlag OutputterObj::eventElementEnd(Sit S, const EQName& name)
 {
-  assert(!(physical && mySAXOutputType == SAXOUTPUT_INT_PHYSICAL));
+  sabassert(!(physical && mySAXOutputType == SAXOUTPUT_INT_PHYSICAL));
   Str physName;
   //  if (S.getProcessor())
   //  temp = S.getProcessor() -> getAliasedName(name, 
@@ -1501,7 +1512,7 @@ eFlag OutputterObj::eventElementEnd(Sit S, const EQName& name)
 	  };
       }; break;
     default:
-      assert(!"eventElementEnd");
+      sabassert(!"eventElementEnd");
     };
   
   // send the event to SAX
@@ -1540,7 +1551,7 @@ eFlag OutputterObj::eventElementEnd(Sit S, const EQName& name)
       break;
     case SAXOUTPUT_NONE: break;
     default:
-      assert(!"eventElementEnd");
+      sabassert(!"eventElementEnd");
     };
   
   //remove our namespaces
@@ -1580,7 +1591,8 @@ eFlag OutputterObj::eventAttributeStart(Sit S, const EQName& name)
 	    };
         break;
     default:
-        assert(!"eventAttributeStart");
+      Err1(S, E1_ATTRIBUTE_MISPLACED, fullName);
+      //sabassert(!"eventAttributeStart");
     };
     return OK;
 }
@@ -1588,7 +1600,7 @@ eFlag OutputterObj::eventAttributeStart(Sit S, const EQName& name)
 
 eFlag OutputterObj::eventAttributeEnd(Sit S)
 {
-    assert(state == STATE_IN_ATTRIBUTE);
+    sabassert(state == STATE_IN_ATTRIBUTE);
 
     int currAttNameNdx = currAtts.findNdx(currAttName);
     if (currAttNameNdx != -1)
@@ -1615,14 +1627,14 @@ eFlag OutputterObj::eventCommentStart(Sit S)
             state = STATE_IN_COMMENT;
         }; break;
     default:
-        assert(!"eventCommentStart");
+        sabassert(!"eventCommentStart");
     };
     return OK;
 }
 
 eFlag OutputterObj::eventCommentEnd(Sit S)
 {
-    assert(state == STATE_IN_COMMENT);
+    sabassert(state == STATE_IN_COMMENT);
     if (physical && method == OUTPUT_UNKNOWN)
         E( front.appendConstruct(S, FM_COMMENT, currData, 
                                  ""/**theEmptyString*/, FALSE) )
@@ -1651,14 +1663,14 @@ eFlag OutputterObj::eventPIStart(Sit S, const Str& name)
             currPIName = name;
         }; break;
     default:
-        assert(!"eventPIStart");
+        sabassert(!"eventPIStart");
     };
     return OK;
 }
 
 eFlag OutputterObj::eventPIEnd(Sit S)
 {
-    assert(state == STATE_IN_PI);
+    sabassert(state == STATE_IN_PI);
 
     //check te PI data
     if ( strstr((char*)currData, "?>") )
@@ -1680,7 +1692,7 @@ eFlag OutputterObj::eventPIEnd(Sit S)
 eFlag OutputterObj::eventNamespace(Sit S, const Str& prefix, const Str& uri,
 				   Bool hidden)
 {
-    assert(state == STATE_IN_MARKUP);
+    sabassert(state == STATE_IN_MARKUP);
     // GP: OK, not allocated
     Str search = uri;
     int existing_ndx = currNamespaces.findNum(prefix);
@@ -1714,7 +1726,6 @@ eFlag OutputterObj::eventNamespace(Sit S, const Str& prefix, const Str& uri,
     else
       {
 	Bool exIsHidden = currNamespaces[existing_ndx]->hidden;
-	// the following condition was asserted here for some reason
 	if (!(*existing_namespace == uri))
 	  {
 	    if (existing_ndx >= getFirstOwnNS())
@@ -1780,7 +1791,6 @@ eFlag OutputterObj::eventData(Sit S, const Str& data, Bool hardCData /* = FALSE
             if (!(getFlags() & CDATA_SECTION) && !hardCData)
             {
                 int htmlScriptOrStyle = ( getFlags() & HTML_SCRIPT_OR_STYLE );
-
                 if (physical)
                     E( physical -> outputText(S, data, !outputEscaping, 
 					      htmlScriptOrStyle) );
@@ -1797,7 +1807,7 @@ eFlag OutputterObj::eventData(Sit S, const Str& data, Bool hardCData /* = FALSE
             currData += data;
         }; break;
     default:
-        assert(!"eventData()");
+        sabassert(!"eventData()");
     }
     return OK;
 }
@@ -1817,14 +1827,14 @@ eFlag OutputterObj::eventCDataSection(Sit S, const Str& data)
 		    E( reportCurrData(S, /* hardCData = */ TRUE) );
 		}; break;
 		default:
-		    assert(!"eventCDataSection()");
+		    sabassert(!"eventCDataSection()");
     };    
     return OK;
 }
 
-eFlag OutputterObj::eventEndOutput(Sit S)
+eFlag OutputterObj::eventEndOutput(Sit S, Bool closePhysical /* = false */)
 {
-    assert(state == STATE_OUTSIDE);
+    sabassert(state == STATE_OUTSIDE);
     E( reportCurrData(S) );
     if (physical && method == OUTPUT_UNKNOWN)
     {
@@ -1839,12 +1849,18 @@ eFlag OutputterObj::eventEndOutput(Sit S)
     //pop fake history item
     history.freelast(FALSE);
 
+    //close physical (used for multiple doc output)
+    if (physical && closePhysical) 
+      {
+	E( physical -> close(S) );
+      }
+    
     return OK;
 }
 
 eFlag OutputterObj::eventTrailingNewline(Sit S)
 {
-  assert(state == STATE_OUTSIDE);
+  sabassert(state == STATE_OUTSIDE);
   if (physical)
     {
       E(physical -> outputTrailingNewline(S))
@@ -1876,7 +1892,7 @@ eFlag OutputterObj::reportCurrData(Sit S, Bool hardCData /* = FALSE */)
             }
         }; break;
     default:
-        assert(!"reportCurrData()");
+        sabassert(!"reportCurrData()");
     }
     currData.empty();
     return OK;
@@ -1890,7 +1906,7 @@ eFlag OutputterObj::reportCurrData(Sit S, Bool hardCData /* = FALSE */)
 
 eFlag OutputterObj::reportStartTag(Sit S, Bool isEmpty)
 {
-  assert(!(physical && mySAXOutputType == SAXOUTPUT_INT_PHYSICAL));
+  sabassert(!(physical && mySAXOutputType == SAXOUTPUT_INT_PHYSICAL));
   // FIXME: couldn't improve reportStartTag from GP point of view?
   if (state == STATE_OUTSIDE || currElement.isEmpty())
     return OK;
@@ -2008,9 +2024,9 @@ eFlag OutputterObj::reportStartTag(Sit S, Bool isEmpty)
 	  IF_SAX3(startElement, physName, attsTable);
 	  break;
 	case SAXOUTPUT_NONE:
-	  assert(!"saxoutput == NONE");
+	  sabassert(!"saxoutput == NONE");
 	default:
-	  assert(!"eventElementEnd");
+	  sabassert(!"eventElementEnd");
 	};
       //attsTable.delArray();	
     }
